import 'dart:async';

import '../ftp_reply.dart';
import '../ftp_socket.dart';
import '../ftpclient_base.dart';

class FTPFile {
  final FTPSocket _socket;

  FTPFile(this._socket);

  Future<bool> rename(String sOldName, String sNewName) async {
    FTPReply sResponse = await (_socket.sendCommand('RNFR $sOldName'));
    if (sResponse.code != 350) {
      return false;
    }

    sResponse = await (_socket.sendCommand('RNTO $sNewName'));
    return sResponse.isSuccessCode();
  }

  Future<bool> delete(String? sFilename) async {
    FTPReply sResponse = await (_socket.sendCommand('DELE $sFilename'));

    return sResponse.isSuccessCode();
  }

  Future<bool> exist(String sFilename) async {
    return await size(sFilename) != -1;
  }

  Future<int> size(String? sFilename) async {
    try {
      FTPReply sResponse = await (_socket.sendCommand('SIZE $sFilename'));
      if (!sResponse.isSuccessCode()) {
        //check if ascii mode get refused
        //change to binary mode if ascii mode refused
        await _socket.setTransferType(TransferType.binary);
        sResponse = await (_socket.sendCommand('SIZE $sFilename'));
        //back to ascci mode
        await _socket.setTransferType(TransferType.ascii);
      }
      return int.parse(sResponse.message.replaceAll('213 ', ''));
    } catch (e) {
      return -1;
    }
  }
}
